/**
 * @(#)Rect.java  12/12/99
 * @author Peter Au
 * All rights reserved.
 */

package vishnu.testvis.treemap.rectangle;

import java.awt.geom.Rectangle2D;
import java.awt.*;

/**
 * The Rect class contains direction and colour, the direction is calculated
 * based on the initial points.
 */
abstract class Rect extends Rectangle2D.Float{
//-- the rectangle will move to this direction during zooming
final static protected int __NORTH=0, __EAST=1, __SOUTH=2, __WEST=3;
private int _direction;
//-- any two points to create a rectangle
private float _left_x, _left_y, _right_x, _right_y;
//-- color of the rectangle
private Color _colour;

  public Rect(float x1, float y1, float x2, float y2, Color c){
    setup(x1, y1, x2, y2);
    _colour = c;
  }


  public void showInfo(){
    System.err.println( "x: "+getX() +"  y: "+getY()+"  w: "+getWidth()+"  h: "+getHeight());
    System.err.println(   "    direction: "+getDirection_toString()+"  colour: "+
        _colour.toString());
  }


  /**
   * setup the direction and the corresponding coordination
   */
  private void setup(float x1, float y1, float x2, float y2){
    _left_x=x1; _left_y=y1;
    _right_x=x2; _right_y=y2;
    //-- set up the direction
    if (x1<x2)
      if (y1<y2) _direction = __WEST;
      else if (y1>y2) _direction = __SOUTH;
      else System.err.println("Error - Rectangle box cannot be zero");
    else if (x1>x2)
      if (y1<y2) _direction = __NORTH;
      else if (y1>y2) _direction = __EAST;
      else System.err.println("Error - Rectangle box cannot be zero");
    else
      System.err.println("Error - Rectangle box cannot be zero");
    //-- setup the corresponding coordination of the rectangle obj
    switch (_direction) {
      case __NORTH:
        super.setRect(x2,y1,x1-x2,y2-y1);
        break;
      case __EAST:
        super.setRect(x2,y2,x1-x2,y1-y2);
        break;
      case __SOUTH:
        super.setRect(x1,y2,x2-x1,y1-y2);
        break;
      case __WEST:
        super.setRect(x1,y1,x2-x1,y2-y1);
        break;
    }
  }  //-- setup


  public void paint(Graphics oldg){
    Graphics2D g = (Graphics2D) oldg;
    g.setColor(getColor());
    g.fill( getBounds2D() );
    g.setColor(Color.black);
    g.draw( getBounds2D() );
  }

  public void paintBorder(Graphics oldg, float w){
    Graphics2D g = (Graphics2D) oldg;
    g.fill(new Rectangle2D.Float(getUpperLeftX(),getUpperLeftY(),
       w, (float) getHeight()));
    g.fill(new Rectangle2D.Float(getLowerRightX()-w,getUpperLeftY(),
       w, (float) getHeight()));
    g.fill(new Rectangle2D.Float(getUpperLeftX(),getUpperLeftY(),
       (float) getWidth(),w));
    g.fill(new Rectangle2D.Float(getUpperLeftX(),getLowerRightY()-w,
       (float) getWidth(),w));
  }

  public boolean isNorth(){return _direction==__NORTH;}
  public boolean isEast(){return _direction==__EAST;}
  public boolean isSouth(){return _direction==__SOUTH;}
  public boolean isWest(){return _direction==__WEST;}

  public Color getColor(){return _colour;}
  public float getUpperLeftX(){return ((float) getX());}
  public float getUpperLeftY(){return ((float) getY());}
  public float getLowerRightX(){return ((float) (getX()+getWidth()));}
  public float getLowerRightY(){return ((float) (getY()+getHeight()));}
  public int getDirection(){return _direction;}
  public int getOppositeDirection(){return (_direction+2)%4;}

  /**
   * @return the direction of the rectangule as a string
   */
  public String getDirection_toString(){
    switch (_direction){
      case __NORTH: return "NORTH";
      case __EAST: return "EAST";
      case __SOUTH: return "SOUTH";
      default: return "WEST";
    }
  }


}  //-- Rect
